from django.test import SimpleTestCase
from django.http import HttpRequest
from django.template import Template, Context
from django.core.paginator import Paginator

from django_cool_paginator.templatetags import paginator_tags
from django_cool_paginator.exceptions import PageNotSpecified, RequestNotExists


class BaseTest(SimpleTestCase):

    def setUp(self):
        paginator = Paginator([... for _ in range(30)], 5)
        self.page = paginator.page(4)
        self.request = HttpRequest()

        self.base_context = {
            'request': self.request,
            'page_obj': paginator.page(4),
        }

        self.size_conf = {
            'LARGE': 'pagination-lg',
            'SMALL': 'pagination-sm'
        }


class PaginatorTagTest(BaseTest):
    """These tests are designed for paginator_tags.py internal module which is in templatetags"""

    load = '{% load paginator_tags %}'

    def test_ellipsis_or_number(self):
        template_string = Template(self.load + '{% ellipsis_or_number page_obj.paginator page_numb %}')

        self.request.GET['page'] = 5

        context = self.base_context.copy()
        context['page_numb'] = 5

        template = template_string.render(Context(context))
        self.assertEqual(int(template), 5)

        context['page_numb'] = 8

        template = template_string.render(Context(context))
        self.assertEqual(template, '...')

        self.request.GET['page'] = 1
        context['page_numb'] = 7

        template = template_string.render(Context(context))
        self.assertEqual(template, str(None))

    def test_url_replace(self):
        template_string = Template(self.load + "{% url_replace 'page' number %}")

        context = self.base_context.copy()
        context['number'] = 2

        template = template_string.render(Context(context))
        self.assertEquals(template, 'page=2')

    def test_size(self):
        template_string = Template(self.load + '{% size size %}')

        context = {'size': 'LARGE'}

        template = template_string.render(Context(context))
        self.assertEqual(template, self.size_conf[context['size']])

        #                               By default
        template = template_string.render(Context())
        self.assertEqual(template, self.size_conf[paginator_tags.COOL_PAGINATOR_SIZE])

    def test_previous_name(self):
        template_string = Template(self.load + "{% previous_name name=previous_name %}")

        previous_name = 'Bobby'

        template = template_string.render(Context({'previous_name': previous_name}))
        self.assertEqual(template, previous_name)

        #                               By default
        template = template_string.render(Context())
        self.assertEqual(template, paginator_tags.COOL_PAGINATOR_PREVIOUS_NAME)

    def test_next_name(self):
        template_string = Template(self.load + "{% next_name name=next_name %}")

        next_name = 'Jack'

        template = template_string.render(Context({'next_name': next_name}))
        self.assertEqual(template, next_name)

        #                               By default
        template = template_string.render(Context({}))
        self.assertEqual(template, paginator_tags.COOL_PAGINATOR_NEXT_NAME)


class CoolPaginateTest(BaseTest):
    """These tests are designed for cool_paginate function which is in cool_paginate.py module"""

    load = '{% load cool_paginate %}'

    def test_page(self):
        template_string = Template(self.load + '{% cool_paginate %}')
        self.assertTrue(template_string.render(Context(self.base_context.copy())))

    def test_exception(self):
        template_string = Template(self.load + '{% cool_paginate %}')

        with self.assertRaisesMessage(RequestNotExists,
                                      'Unable to find request in your template context,'
                                      'please make sure that you have the request context processor enabled'
                                      ):
            template_string.render(Context())

        with self.assertRaisesMessage(PageNotSpecified,
                                      'You customized paginator standard name, '
                                      "but haven't specified it in {% cool_paginate %} tag."
                                      ):
            template_string.render(Context({'request': self.request}))

    def test_size(self):

        context = self.base_context.copy()
        context['size'] = 'LARGE'

        template_string = Template(self.load + '{% cool_paginate size=size %}')
        template = template_string.render(Context(context))

        self.assertIn(self.size_conf[context['size']], template)

        #                               By default
        context.pop('size')
        template = template_string.render(Context(context))

        self.assertIn(self.size_conf[paginator_tags.COOL_PAGINATOR_SIZE], template)

    def test_next_name(self):

        context = self.base_context.copy()
        context['next_name'] = 'Go to'

        template_string = Template(self.load + '{% cool_paginate next_name=next_name %}')
        template = template_string.render(Context(context))

        self.assertIn(context['next_name'], template)

        #                               By default
        context.pop('next_name')
        template = template_string.render(Context(context))
        self.assertIn(paginator_tags.COOL_PAGINATOR_NEXT_NAME, template)

    def test_previous_name(self):

        context = self.base_context.copy()
        context['previous_name'] = 'Back to'

        template_string = Template(self.load + '{% cool_paginate previous_name=previous_name %}')
        template = template_string.render(Context(context))

        self.assertIn(context['previous_name'], template)

        #                               By default
        context.pop('previous_name')
        template = template_string.render(Context(context))
        self.assertIn(paginator_tags.COOL_PAGINATOR_PREVIOUS_NAME, template)

    def test_elastic(self):

        context = self.base_context.copy()
        context['elastic'] = 5

        standard_template = Template(self.load + '{% cool_paginate elastic=elastic%}').render(Context(context)).strip()

        context['elastic'] = 10
        template = Template(self.load + '{% cool_paginate elastic=elastic%}').render(Context(context)).strip()

        self.assertHTMLNotEqual(template, standard_template)



import os.path

BASE_DIR = os.path.dirname(os.path.dirname(os.path.abspath(__file__)))


SECRET_KEY = 'fake-key'

INSTALLED_APPS = [
    'django_cool_paginator',

]

TEMPLATES = [
    {
        'BACKEND': 'django.template.backends.django.DjangoTemplates',
        'APP_DIRS': True,
    }
]

# For testing purposes.

COOL_PAGINATOR_SIZE = 'SMALL'
COOL_PAGINATOR_NEXT_NAME = 'Go'
COOL_PAGINATOR_PREVIOUS_NAME = 'Previous'



"""
Tags for pagination template which is templates directory

These are internal pagination custom tag functions that are needed by paginator template (__paginator.html)
which is in templates directory. User doesn't have to interact with them. They are designed as internal.

Most of them depend on project settings.

Items:
    Vars:
        - COOL_PAGINATOR_NEXT_NAME
        - COOL_PAGINATOR_PREVIOUS_NAME
        - COOL_PAGINATOR_SIZE

    Functions:
        - url_replace(context, field, value)
        - ellipsis_or_number(context, paginator, current_page)
        - size(chosen_size=None)
        - next_name(name=None)
        - previous_name(name=None)

Description:
    Function description:

        url_replace:
            To avoid GET params loosing
        ellipsis_or_number:
            To avoid display a long page table
        size:
            Points to pagination table size.

"""

from django import template
from django.conf import settings


register = template.Library()


#                                                     Pagination settings

COOL_PAGINATOR_NEXT_NAME = getattr(settings, 'COOL_PAGINATOR_NEXT_NAME', 'Next')
COOL_PAGINATOR_PREVIOUS_NAME = getattr(settings, 'COOL_PAGINATOR_PREVIOUS_NAME', 'Previous')
COOL_PAGINATOR_SIZE = getattr(settings, 'COOL_PAGINATOR_SIZE', None)

#                                                           Tags...


# next_name tag
next_name = register.simple_tag(name='next_name', func=lambda name=None: name or COOL_PAGINATOR_NEXT_NAME)

# previous_name tag
previous_name = register.simple_tag(name='previous_name', func=lambda name=None: name or COOL_PAGINATOR_PREVIOUS_NAME)


@register.simple_tag(takes_context=True)
def url_replace(context, field, value):
    """
    To avoid GET params losing

    :param context: context_obj
    :param field: str
    :param value: str
    :return: dict-like object
    """

    query_string = context['request'].GET.copy()
    query_string[field] = value

    return query_string.urlencode()


# TODO Make this function maximum customizable
@register.simple_tag(takes_context=True)
def ellipsis_or_number(context, paginator, current_page):
    """
    To avoid display a long pagination bar

    :param context: template context
    :param paginator: paginator_obj
    :param current_page: int
    :return: str or None
    """

    # Checks is it first page
    chosen_page = int(context['request'].GET['page']) if 'page' in context['request'].GET else 1

    if current_page in (chosen_page + 1, chosen_page + 2, chosen_page - 1, chosen_page - 2,
                        paginator.num_pages, paginator.num_pages - 1, 1, 2, chosen_page):
        return current_page

    if current_page in (chosen_page + 3, chosen_page - 3):
        return '...'


@register.simple_tag
def size(chosen_size=None):
    """
    Points to pagination bar size.

    :param chosen_size: int
    :return: str
    """

    return {
        'LARGE': 'pagination-lg',
        'SMALL': 'pagination-sm',
    }.get(chosen_size or COOL_PAGINATOR_SIZE, '')



__author__ = 'Jabrail Lezgintsev'
__credits__ = ['Jabrail Lezgintsev']
__license__ = 'MIT'
__version__ = '0.3.0'
__maintainer__ = 'Jabrail Lezgintsev'
__email__ = 'lezgintsev13@yandex.ru'
__status__ = 'Beta'



"""
These exceptions relate to django-cool-pagination.

Items:
    Exceptions:
        - PageNotSpecified(*args, **kwargs)
        - RequestNotExists(*args, **kwargs)

Description:
    Exception Description:

        PageNotSpecified:
            Raises in case if page wasn't specified at all

        RequestNotExists:
            Raises in case if user has forgotten to include request context processor

"""


class PageNotSpecified(Exception):
    """Raises in case if page wasn't specified at all"""
    pass


class RequestNotExists(Exception):
    """Raises in case if user has forgotten to include request context processor"""
    pass



import os

from setuptools import find_packages, setup

with open(os.path.join(os.path.dirname(__file__), 'README.md')) as readme:
    README = readme.read()

os.chdir(os.path.normpath(os.path.join(os.path.abspath(__file__), os.pardir)))

setup(
    name='django-cool-pagination',
    version='0.3.0',
    packages=find_packages(),
    include_package_data=True,
    license='MIT',
    description='Simple pagination app that saves your time.',
    long_description_content_type='text/markdown',
    long_description=README,
    url='https://github.com/joe513/django-cool-pagination',
    author='Jabrail Lezgintsev',
    author_email='lezgintsev13@yandex.ru',
    keywords='python3 django pagination bootstrap4',
    project_urls={
        'Source': 'https://github.com/joe513/django-cool-pagination'
    },
    python_requires='>=3',
    classifiers=[
        'Environment :: Web Environment',
        'Framework :: Django',
        'Framework :: Django :: 2.0',
        'Intended Audience :: Developers',
        'License :: OSI Approved :: MIT License',
        'Operating System :: OS Independent',
        'Programming Language :: Python',
        'Programming Language :: Python :: 3.6',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: 3.4',
        'Topic :: Internet :: WWW/HTTP',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
    ],
)



"""
Main module for pagination purpose

This module is designed as main. To load it to your template just write:

                        {% load cool_paginate %}

Items:
    Functions:

        - cool_paginate(context, **kwargs)

Description:
    Function description:

        cool_paginate:
            Main function for pagination process.

"""

from django import template

from django_cool_paginator.exceptions import PageNotSpecified, RequestNotExists
from django.conf import settings

# TODO Discuss which name is better for the module cool_paginate or cool_pagination

register = template.Library()


@register.inclusion_tag('__paginator.html', takes_context=True)
def cool_paginate(context, **kwargs):
    """Main function for pagination process."""

    names = (
        'size',
        'next_name',
        'previous_name',
        'elastic',
        'page_obj',
    )

    return_dict = {name: value for name, value in zip(names, map(kwargs.get, names))}

    if context.get('request'):
        return_dict['request'] = context['request']
    else:
        raise RequestNotExists(
            'Unable to find request in your template context,'
            'please make sure that you have the request context processor enabled'
        )

    if not return_dict.get('page_obj'):
        if context.get('page_obj'):
            return_dict['page_obj'] = context['page_obj']
        else:
            raise PageNotSpecified(
                'You customized paginator standard name, '
                "but haven't specified it in {% cool_paginate %} tag."
            )

    if not return_dict.get('elastic'):
        return_dict['elastic'] = getattr(settings, 'COOL_PAGINATOR_ELASTIC', 10)

    return return_dict
